# encoding: utf8
import re
from decimal import Decimal, InvalidOperation


class FieldValidator(object):
    message = None

    def __init__(self):
        if self.message is None:
            self.message = self.__class__.__name__

    def init_field(self, field):
        self.field = field

    def make_field(self):
        if self.validate_field() is False:
            self.set_field_error()

    def make_value(self, field_value):
        self.field_value = field_value
        self.value = field_value.value
        if self.validate_value() is False:
            self.set_value_error()

    def set_field_error(self):
        self.field.set_error(self.message)

    def set_value_error(self):
        self.field_value.set_error(self.message)

    def validate_field(self):
        pass


class NotEmpty(FieldValidator):

    """Will fail if no value found of value is empty or has only whitespaces"""

    def validate_field(self):
        return len(self.field.values) > 0

    def validate_value(self):
        if self.value is None:
            return False
        elif type(self.value) == str and self.value.strip() == '':
            return False
        elif type(self.value) == bytes and self.value.strip() == b'':
            return False
        elif type(self.value) in [list, dict, tuple] and len(self.value) == 0:
            return False
        return True


class IsDigit(FieldValidator):

    """Will fail if value is not a digit."""

    regex = re.compile('^-{0,1}[0-9]+$')

    def validate_value(self):
        if not self.value:
            return True
        return re.search(self.regex, self.value) is not None


class IsDecimal(FieldValidator):

    """Will fail if value can not be converted to decimal.Decimal."""

    def validate_value(self):
        try:
            Decimal(self.value)
            return True
        except InvalidOperation:
            return False


class IsEmail(FieldValidator):

    """Will fail if value is not an email."""

    regex = re.compile(
        "^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$")

    def validate_value(self):
        if len(self.value) > 7:
            return re.match(self.regex, self.value) is not None
        return False


class IsValueInAvalibleValues(FieldValidator):

    """Will fail if value is not in list."""

    def __init__(self, allow_empty=False):
        super().__init__()
        self.allow_empty = allow_empty

    def validate_value(self):
        print('a', self.value)
        if self.allow_empty:
            print('\t', type(self.value) == str)
            if type(self.value) == str:
                print('\t',  self.value.strip() == '')
            if self.value is None:
                return True
            elif type(self.value) == str and self.value.strip() == '':
                return True
        return self.value in self.field.avalible_values



from mock import create_autospec
from unittest import TestCase
from pytest import raises

from formskit.field import Field
from formskit.form import Form, WrongValueName
from formskit.validators import NotEmpty, IsDigit
from formskit.converters import ToInt
from formskit.formvalidators import FormValidator
from formskit.translation import Translation


class FormTest(TestCase):

    def test_add_field_object(self):
        field = create_autospec(Field('name'))
        form = Form()
        form.add_field_object(field)

        field.init_form.assert_called_once_with(form)
        assert field is form.fields[field.name]

    def test_add_field(self):
        form = Form()
        form.add_field('myname')

        assert isinstance(form.fields['myname'], Field)

    def test_reset(self):
        field = create_autospec(Field('name'))
        form = Form()
        form.add_field_object(field)

        form.success = False

        form.reset()

        assert form.success is None
        field.reset.assert_called_once_with()

    def test_is_validated(self):
        form = Form()

        data = {
            form.form_name_value: [form.get_name(), ],
        }
        assert form.validate(data) is True

    def test_is_not_submited(self):
        form = Form()

        assert form.validate({}) is None

    def test_if_not_validated(self):
        form = Form()
        form.add_field('name', validators=[NotEmpty()])

        data = {
            form.form_name_value: [form.get_name(), ],
        }
        assert form.validate(data) is False

    def test_parse_test_data(self):
        form = Form()
        form.add_field('name')
        form.add_field('second')

        form.parse_dict({
            'name': ['n1'],
            'second': 'n2',
        })

        assert form.get_value('name') == 'n1'
        assert form.get_value('second') == 'n2'

    def test_messages(self):
        class ExampleTranslation(Translation):

            def translate(self):
                return 'translated!'
        form = Form()
        form.translation_class = ExampleTranslation

        form.add_field('name', validators=[NotEmpty()])

        assert form.validate({
            form.form_name_value: [form.get_name()],
        }) is False

        assert form.fields['name'].messages[0]() == 'translated!'


class TestGetAndSet(TestCase):

    def setUp(self):
        super().setUp()
        self.form = Form()
        self.form.add_field('name')

    def test_set_once(self):
        self.form.set_value('name', 'value')

        assert self.form.get_value('name') == 'value'
        assert self.form.get_values('name') == ['value']

    def test_set_once_with(self):
        self.form.set_value('name', 'value', 1)
        self.form.set_value('name', 'value2', 1)
        self.form.set_value('name', 'value3', 1)

        assert self.form.get_value('name', 0) == 'value'
        assert self.form.get_value('name', 1) == 'value3'
        assert self.form.get_values('name') == ['value', 'value3']

    def test_set_many(self):
        self.form.set_values('name', ['one', 'two'])

        assert self.form.get_value('name', 0) == 'one'
        assert self.form.get_value('name', 1) == 'two'
        assert self.form.get_values('name') == ['one', 'two']

    def test_set_many_with_converter(self):
        self.form.add_field('name2', convert=ToInt())
        self.form.set_values('name2', ['2', '3'])

        assert self.form.get_value('name2', 0) == 2
        assert self.form.get_value('name2', 1) == 3
        assert self.form.get_values('name2') == [2, 3]

    def test_ignore(self):
        self.form.fields['name'].ignore = True

        self.form.set_value('name', 'one')
        self.form.set_values('name', ['one', 'two'])

        assert self.form.get_values('name') == []

    def test_force(self):
        self.form.fields['name'].ignore = True

        self.form.set_value('name', 'one', force=True)

        assert self.form.get_values('name') == ['one']

    def test_force_many(self):
        self.form.fields['name'].ignore = True

        self.form.set_values('name', ['one', 'two'], force=True)

        assert self.form.get_values('name') == ['one', 'two']


class IsFormSubmittedTest(TestCase):

    def test_name_do_not_match(self):
        form = Form()
        raw_data = {
            Form.form_name_value: [form.get_name() + 'bad'],
        }

        assert form._is_form_submitted(raw_data) is False

    def test_name_match(self):
        form = Form()
        raw_data = {
            Form.form_name_value: [form.get_name()],
        }

        assert form._is_form_submitted(raw_data) is True

    def test_not_form_data(self):
        form = Form()
        raw_data = {}

        assert form._is_form_submitted(raw_data) is False


class ParseRawDataTest(TestCase):

    def setUp(self):
        self.form = Form()
        self.field = self.form.add_field('one')

    def test_good_data(self):
        raw_data = {
            self.form.fields['one'].get_name(): ['value']
        }

        self.form._parse_raw_data(raw_data)

        assert self.form.fields['one'].values[0].value == 'value'

    def test_too_many_data(self):
        raw_data = {
            self.form.fields['one'].get_name(): ['value'],
            'two': ['something'],
        }

        with raises(WrongValueName):
            self.form._parse_raw_data(raw_data)

    def test_too_few_data(self):
        raw_data = {}

        self.form._parse_raw_data(raw_data)

        assert self.form.fields['one'].values == []


class GetDataDictTest(TestCase):

    def setUp(self):
        super().setUp()
        self.form = Form()
        self.form.add_field('name1')
        self.form.add_field('name2')

        data = {
            self.form.form_name_value: [self.form.get_name()],
            'name1': ['one'],
            'name2': ['two', 'three'],
        }

        self.form.validate(data)

    def test_dict(self):
        assert self.form.get_data_dict() == {
            'name1': ['one'],
            'name2': ['two', 'three'],
        }

    def test_dict_minified(self):
        assert self.form.get_data_dict(True) == {
            'name1': 'one',
            'name2': ['two', 'three'],
        }


class ExampleFormValidator(FormValidator):

    message = 'example validator'

    def __init__(self):
        super().__init__()
        self._validate = True

    def validate(self):
        return self._validate


class GetReportTest(TestCase):

    def setUp(self):
        super().setUp()
        self.form = Form()
        self.form.add_field('name1', validators=[NotEmpty()])
        self.form.add_field('name2', validators=[IsDigit()])
        self.form_validator = ExampleFormValidator()
        self.form.add_form_validator(self.form_validator)

    def run_form(self):
        data = {
            self.form.form_name_value: [self.form.get_name()]
        }
        return self.form.validate(data)

    def test_success(self):
        self.form.parse_dict({
            'name1': ['one'],
        })
        assert self.run_form() is True

        assert self.form.get_report() == {
            'success': True,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            }
        }

    def test_error_at_form_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
        })
        self.form_validator._validate = False
        assert self.run_form() is False

        assert self.form.get_report() == {
            'success': False,
            'messages': ['example validator'],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            }
        }

    def test_error_at_field_validator_with_compiling(self):
        self.form.parse_dict({
        })
        assert self.run_form() is False

        assert self.form.get_report() == {
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': False,
                    'messages': ['NotEmpty'],
                    'values': []
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            }
        }

    def test_error_at_field_value_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
            'name2': ['three']
        })
        assert self.run_form() is False

        assert self.form.get_report() == {
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': False,
                    'messages': [],
                    'values': [{
                        'value': 'three',
                        'success': False,
                        'messages': ['IsDigit'],
                    }]
                },
            }
        }



from formskit import Form
from formskit.tests.base import FormskitTestCase
from formskit.formvalidators import MustMatch


class Form1(Form):

    name1 = 'something'
    name2 = 'something2'

    def create_form(self):
        self.add_field(self.name1)
        self.add_field(self.name2)

        self.add_form_validator(MustMatch([self.name1, self.name2]))


class MustMatchTest(FormskitTestCase):

    def setUp(self):
        super().setUp()
        self.form = Form1()
        self.field1 = self.form.fields[self.form.name1]
        self.field2 = self.form.fields[self.form.name2]

    def test_success(self):
        self.field1.set_values(['1'])
        self.field2.set_values(['1'])

        assert self.form._validate_form_validators() is True
        assert self.form.messages == []

    def test_fail(self):
        self.field1.set_values(['1'])
        self.field2.set_values(['2'])

        assert self.form._validate_form_validators() is False
        assert self.form.messages[0].text == 'input must be the same!'

    def test_fail_when_number_of_values_do_not_match(self):
        self.field1.set_values(['1'])
        self.field2.set_values([])

        assert self.form._validate_form_validators() is False
        assert self.form.messages[0].text == 'input must be the same!'

    def test_success_when_number_of_values_do_not_match(self):
        self.field1.set_values(['1', '2'])
        self.field2.set_values(['1'])

        assert self.form._validate_form_validators() is True
        assert self.form.messages == []



from json import loads
from base64 import urlsafe_b64decode
from pytest import raises

from formskit.tests.base import FormskitTestCase
from formskit import Field
from formskit.validators import NotEmpty, IsDigit
from formskit.form import Form
from formskit.tree_form import TreeForm
from formskit.field import FieldValue
from formskit.translation import Translation


class ExampleField(Field):

    def _get_message_object(self):
        return Translation()


class FieldTest(FormskitTestCase):

    def test_init_form(self):
        form = 123
        field = ExampleField(None, None, None)
        field.init_form(form)

        self.assertEqual(form, field.form)

    def test_validate(self):
        not_empty = NotEmpty()
        field = ExampleField('name', [not_empty])

        self.assertEqual(False, field.validate())
        self.assertEqual(field, not_empty.field)

        field.reset()
        field.set_values(['green'])

        self.assertEqual(True, field.validate())

    def test_two_validators(self):
        field = ExampleField('name', [NotEmpty(), IsDigit()])

        field.validate()
        self.assertEqual(False, field.validate())

        field.reset()
        field.set_values(['green'])

        self.assertEqual(False, field.validate())

        field.reset()
        field.set_values(['15'])

        self.assertEqual(True, field.validate())

    def test_set_values(self):
        field = ExampleField('name')

        field.set_values(['value'])

        field_value = field.values[0]
        assert 'value' == field_value.value

    def test_set_values_on_ignore(self):
        field = ExampleField('name', ignore=True)

        field.set_values(['value'])

        assert 0 == len(field.values)

    def test_reset(self):
        field = ExampleField('name')
        field.set_values(['val'])
        field.messages = ['msg']
        field.error = True

        field.reset()

        assert field.values == []
        assert field.messages == []
        assert field.error is False

    def test_reset_on_ignore(self):
        field = ExampleField('name', ignore=True)
        field.set_values(['val'], True)
        field.messages = ['msg']
        field.error = True

        field.reset()

        assert field.values[0].value == 'val'
        assert field.messages == ['msg']
        assert field.error is True

    def test_set_error(self):
        field = ExampleField('name')

        field.set_error('msg')

        assert field.error is True
        assert field.messages[0].text == 'msg'

    def test_index_error(self):
        form = Form()
        form.add_field('name')

        with raises(IndexError):
            form.get_value('name')

    def test_defaul(self):
        form = Form()
        form.add_field('name')

        assert form.get_value('name', default='elf') == 'elf'


class TreeFieldTests(FormskitTestCase):

    def test_get_name_tree(self):
        form = TreeForm()
        form2 = TreeForm()
        field = form2.add_field('name1')
        form.add_sub_form(form2)

        raw = field.get_name()

        json = urlsafe_b64decode(raw).decode()
        data = loads(json)

        assert data == {
            'name': 'name1',
            'parents': [
                {'name': 'TreeForm', 'index': None},
                {'name': 'TreeForm', 'index': 0}],
        }

    def test_get_name_tree_with_index(self):
        form = TreeForm()
        form2 = TreeForm()
        form2.add_field('name1')
        form.add_sub_form(form2)

        field = form.get_or_create_sub_form('TreeForm', 2).fields['name1']

        raw = field.get_name()

        json = urlsafe_b64decode(raw).decode()
        data = loads(json)

        assert data == {
            'name': 'name1',
            'parents': [
                {'name': 'TreeForm', 'index': None},
                {'name': 'TreeForm', 'index': 2},
            ],
        }

    def test_get_name(self):
        form = TreeForm()
        field = form.add_field('name1')

        raw = field.get_name()

        json = urlsafe_b64decode(raw).decode()
        data = loads(json)

        assert data == {
            'name': 'name1',
            'parents': [{'name': 'TreeForm', 'index': None}],
        }


class GetValueErrorTests(FormskitTestCase):

    def setUp(self):
        super().setUp()
        self.field = ExampleField('name')

    def test_index_error(self):
        with raises(IndexError):
            self.field.get_value_errors()

    def test_defaul(self):
        assert self.field.get_value_errors(default='elf') == 'elf'

    def test_normal(self):
        self.field.values.append(FieldValue(self.field, 'val'))
        self.field.values[0].messages = [(lambda: 'my error')]

        assert self.field.get_value_errors() == ['my error']



from pytest import fixture

from formskit.tests.base import FormskitTestCase
from formskit.field import Field, FieldValue
from formskit.translation import Translation
import formskit.validators as VAL


class ExampleField(Field):

    def _get_message_object(self):
        return Translation()


class ValidatorTestMixin(object):
    cls = None
    good_samples = None
    bad_samples = None

    def setUp(self):
        super().setUp()
        self.field = ExampleField('name')
        self.validator = self.cls()
        self.validator.init_field(self.field)

    def test_success(self):
        for sample in self.good_samples:
            field_value = FieldValue(self.field, sample)
            self.validator.make_value(field_value)

            assert self.field.error is False
            assert self.field.messages == []
            assert field_value.messages == []
            assert field_value.error is False

    def test_fail(self):
        for sample in self.bad_samples:
            self.field.reset()
            field_value = FieldValue(self.field, sample)

            self.validator.make_value(field_value)

            assert self.field.error is True
            if self._is_field_value_validator():
                assert self.field.messages == []
                assert field_value.error is True
                assert field_value.messages[0].text == self.cls.__name__
            else:
                assert self.field.messages[0].text == self.cls.__name__
                assert field_value.messages == []
                assert field_value.error is False

    def _is_field_value_validator(self):
        return issubclass(self.cls, VAL.FieldValidator)

    def test_set_form(self):
        field = ExampleField('my field')
        validator = self.cls()
        validator.init_field(field)

        self.assertEqual(field, validator.field)


class NotEmptyValidatorTest(ValidatorTestMixin, FormskitTestCase):
    cls = VAL.NotEmpty

    good_samples = [
        'z', '0', '12312312dasd213123', ',', ' ad sda ',
    ]

    bad_samples = [
        ' ', '', None, [], {}, b'',
    ]


class IsDigitValidatorTest(ValidatorTestMixin, FormskitTestCase):
    cls = VAL.IsDigit

    good_samples = [
        '1', '123123123', '012312', '-123123', '',
    ]

    bad_samples = [
        'a', '-123213.12323', '2a',
    ]


class EmailValidatorTest(ValidatorTestMixin, FormskitTestCase):
    cls = VAL.IsEmail

    good_samples = [
        'msocek@gmail.com',
        '1asd@asdasd.sadad.pl',
    ]

    bad_samples = [
        'a', '', '-123213.12323', '2a',
        ' ', '@asdweq.asdad.pl', '1asd@asdasd.sadad.asdpl',
    ]


class IsDecimalTest(ValidatorTestMixin, FormskitTestCase):
    cls = VAL.IsDecimal

    good_samples = [
        '1',
        '1.2',
        '1.2213123123123123',
        '123123123.123123'
        '234234234234',
        '-123123.123123',
    ]

    bad_samples = [
        'a', '', '-123213,12323', '2a',
        ' ', '@asdweq.asdad.pl', '1asd@asdasd.sadad.asdpl',
    ]


class IsValueInAvalibleValuesTest(FormskitTestCase):
    cls = VAL.IsValueInAvalibleValues

    def setUp(self):
        super().setUp()
        self.field = ExampleField('name')
        self.field.set_avalible_values(['5'])
        self.validator = self.cls()
        self.validator.init_field(self.field)

    def test_good(self):
        field_value = FieldValue(self.field, '5')
        self.validator.make_value(field_value)

        assert self.field.error is False
        assert self.field.messages == []
        assert field_value.messages == []
        assert field_value.error is False

    def test_fail(self):
        field_value = FieldValue(self.field, '6')
        self.validator.make_value(field_value)

        assert self.field.error is True
        assert self.field.messages == []
        assert field_value.error is True
        assert field_value.messages[0].text == self.cls.__name__

    def test_good_method(self):
        def method():
            return ['4']

        self.field.set_avalible_values(method)

        field_value = FieldValue(self.field, '4')
        self.validator.make_value(field_value)

        assert self.field.error is False
        assert self.field.messages == []
        assert field_value.messages == []
        assert field_value.error is False

    def test_fail_method(self):
        def method():
            return ['4']

        self.field.set_avalible_values(method)

        field_value = FieldValue(self.field, '5')
        self.validator.make_value(field_value)

        assert self.field.error is True
        assert self.field.messages == []
        assert field_value.error is True
        assert field_value.messages[0].text == self.cls.__name__

    def test_allow_empty_true_for_empty_string(self):
        self.field.set_avalible_values([])
        self.validator.allow_empty = True

        field_value = FieldValue(self.field, '')
        self.validator.make_value(field_value)

        assert self.field.error is False
        assert self.field.messages == []
        assert field_value.messages == []
        assert field_value.error is False

    def test_allow_empty_true_for_none(self):
        self.field.set_avalible_values([])
        self.validator.allow_empty = True

        field_value = FieldValue(self.field, None)
        self.validator.make_value(field_value)

        assert self.field.error is False
        assert self.field.messages == []
        assert field_value.messages == []
        assert field_value.error is False

    def test_allow_empty_true_for_string(self):
        self.field.set_avalible_values([])
        self.validator.allow_empty = True

        field_value = FieldValue(self.field, 'teststr')
        self.validator.make_value(field_value)

        assert self.field.error is True
        assert self.field.messages == []
        assert field_value.error is True
        assert field_value.messages[0].text == self.cls.__name__


class TestFieldValidatorInit(object):

    def test_custom_message(self):
        """
        .__init__ should do nothing when message is set.
        """
        class Example2(VAL.FieldValidator):
            message = 'custome message'

        obj = Example2()

        assert obj.message == 'custome message'

    def test_default_message(self):
        """
        .__init__ should generate message from class name
        """
        class Example3(VAL.FieldValidator):
            pass

        obj = Example3()

        assert obj.message == 'Example3'



from mock import create_autospec
from unittest import TestCase
from pytest import raises

from formskit.field import Field
from formskit.form import WrongValueName
from formskit.tree_form import TreeForm
from formskit.validators import NotEmpty, IsDigit
from formskit.formvalidators import FormValidator


class TreeFormTest(TestCase):

    def test_reset(self):
        field = create_autospec(Field('name'))
        form = TreeForm()
        form.add_field_object(field)
        form2 = create_autospec(TreeForm())
        form2_name = form2.get_name.return_value
        form.add_sub_form(form2)
        form.get_or_create_sub_form(form2_name, 3)

        form.success = False

        form.reset()

        assert form.success is None
        field.reset.assert_called_once_with()
        assert len(form.childs[form2_name]) == 1
        form2.reset.assert_called_once_with()

    def test_parse_test_data(self):
        form = TreeForm()
        form.add_field('name')
        form.add_field('second')
        form2 = TreeForm()
        form2.add_field('surname')
        form.add_sub_form(form2)

        form.parse_dict({
            'name': ['n1'],
            'second': 'n2',
            'TreeForm': [{
                'surname': ['s1']
            }]
        })

        assert form.get_value('name') == 'n1'
        assert form.get_value('second') == 'n2'

    def test_error_decoding_name(self):
        form = TreeForm()
        form.add_field('one')
        name = form.fields['one'].get_name()
        raw_data = {
            name.replace(b'x', b'y'): ['value'],
        }

        with raises(WrongValueName):
            form._parse_raw_data(raw_data)


class TreeFormsTest(TestCase):

    def setUp(self):
        self.form = TreeForm()
        self.field = self.form.add_field('one')

        self.subform = TreeForm()
        self.subform.add_field('two', validators=[NotEmpty()])
        self.form.add_sub_form(self.subform)

    def _get_raw_data(self, value='value3'):
        raw_data = {
            self.form.form_name_value: [self.form.get_name(), ],
            self.form.fields['one'].get_name(): ['value1'],

        }
        name = (
            self.form.get_or_create_sub_form('TreeForm', 2)
            .fields['two'].get_name()
        )
        raw_data[name] = ['value2.1', 'value2.2']
        name = (
            self.form.get_or_create_sub_form('TreeForm', 3)
            .fields['two'].get_name()
        )
        raw_data[name] = [value]

        return raw_data

    def test_parse_tree(self):
        raw_data = self._get_raw_data()

        self.form.validate(raw_data)

        assert self.form.fields['one'].values[0].value == 'value1'

        field = self.form.get_sub_form('TreeForm', 2).fields['two']
        assert field.values[0].value == 'value2.1'
        assert field.values[1].value == 'value2.2'

        field = self.form.get_sub_form('TreeForm', 3).fields['two']
        assert field.values[0].value == 'value3'

    def test_validation(self):
        raw_data = self._get_raw_data('')

        self.form.validate(raw_data)

        assert self.form.success is False

        field = self.form.get_sub_form('TreeForm', 2).fields['two']
        assert field.error is False

        field = self.form.get_sub_form('TreeForm', 3).fields['two']
        assert field.error is True


class GetDataDictTreeTest(TestCase):

    def setUp(self):
        super().setUp()
        self.form = TreeForm()
        self.form.add_field('name1')

        subform = TreeForm()
        subform.add_field('name2')

        self.form.add_sub_form(subform)

        data = {
            self.form.form_name_value: [self.form.get_name()],
            self.form.fields['name1'].get_name(): ['one'],
            (
                self.form.get_or_create_sub_form('TreeForm', 0)
                .fields['name2'].get_name()
            ): [
                'two',
                'three'],
            (
                self.form.get_or_create_sub_form('TreeForm', 1)
                .fields['name2'].get_name()
            ): [
                'four'],
            (
                self.form.get_or_create_sub_form('TreeForm', 2)
                .fields['name2'].get_name()
            ): [],
        }

        self.form.validate(data)

    def test_tree(self):
        assert self.form.get_data_dict() == {
            'name1': ['one'],
            'TreeForm': {
                0: {'name2': ['two', 'three']},
                1: {'name2': ['four']},
                2: {'name2': []},
            }
        }

    def test_tree_minified(self):
        assert self.form.get_data_dict(True) == {
            'name1': 'one',
            'TreeForm': {
                0: {'name2': ['two', 'three']},
                1: {'name2': 'four'},
                2: {},
            }
        }


class GetReportTreeTest(TestCase):

    def setUp(self):
        super().setUp()
        self.form = TreeForm()
        self.form.add_field('name1', validators=[NotEmpty()])
        self.form.add_field('name2', validators=[IsDigit()])
        self.form_validator = ExampleFormValidator()
        self.form.add_form_validator(self.form_validator)
        self.child = TreeForm()
        self.child.add_field('name1', validators=[NotEmpty()])
        self.child.add_field('name2', validators=[IsDigit()])
        self.child_validator = ExampleFormValidator()
        self.child.add_form_validator(self.child_validator)
        self.form.add_sub_form(self.child)

    def run_form(self):
        data = {
            self.form.form_name_value: [self.form.get_name()]
        }
        return self.form.validate(data)

    def test_success(self):
        self.form.parse_dict({
            'name1': ['one'],
            'TreeForm': [
                {
                    'name1': ['two'],
                }
            ]
        })
        assert self.run_form() is True

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': [],
                                'success': True,
                                'values': [
                                    {
                                        'messages': [],
                                        'success': True,
                                        'value': 'two'
                                    }
                                ]
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': [],
                        'success': True
                    }
                }
            },
            'success': True,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            },
        }

    def test_error_at_form_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
            'TreeForm': [
                {
                    'name1': ['two'],
                }
            ]
        })
        self.form_validator._validate = False
        assert self.run_form() is False

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': [],
                                'success': True,
                                'values': [
                                    {
                                        'messages': [],
                                        'success': True,
                                        'value': 'two'
                                    }
                                ]
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': [],
                        'success': True
                    }
                }
            },
            'success': False,
            'messages': ['example validator'],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            },
        }

    def test_error_at_field_validator_with_compiling(self):
        self.form.parse_dict({
            'TreeForm': [
                {
                    'name1': ['two'],
                }
            ]
        })
        assert self.run_form() is False

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': [],
                                'success': True,
                                'values': [
                                    {
                                        'messages': [],
                                        'success': True,
                                        'value': 'two'
                                    }
                                ]
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': [],
                        'success': True
                    }
                }
            },
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': False,
                    'messages': ['NotEmpty'],
                    'values': []
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            },
        }

    def test_error_at_field_value_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
            'name2': ['three'],
            'TreeForm': [
                {
                    'name1': ['two'],
                }
            ]
        })
        assert self.run_form() is False

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': [],
                                'success': True,
                                'values': [
                                    {
                                        'messages': [],
                                        'success': True,
                                        'value': 'two'
                                    }
                                ]
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': [],
                        'success': True
                    }
                }
            },
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': False,
                    'messages': [],
                    'values': [{
                        'value': 'three',
                        'success': False,
                        'messages': ['IsDigit'],
                    }]
                },
            },
        }

    def test_subform_error_at_form_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
            'TreeForm': [
                {
                    'name1': ['two'],
                }
            ]
        })
        self.child_validator._validate = False
        assert self.run_form() is False

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': [],
                                'success': True,
                                'values': [
                                    {
                                        'messages': [],
                                        'success': True,
                                        'value': 'two'
                                    }
                                ]
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': ['example validator'],
                        'success': False
                    }
                }
            },
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            },
        }

    def test_subform_error_at_field_value_validator(self):
        self.form.parse_dict({
            'name1': ['one'],
            'TreeForm': [
                {
                    'name1': [],
                }
            ]
        })
        assert self.run_form() is False

        assert self.form.get_report() == {
            'childs': {
                'TreeForm': {
                    0: {
                        'childs': {},
                        'fields': {
                            'name1': {
                                'messages': ['NotEmpty'],
                                'success': False,
                                'values': []
                            },
                            'name2': {
                                'messages': [],
                                'success': True,
                                'values': []
                            }
                        },
                        'messages': [],
                        'success': False
                    }
                }
            },
            'success': False,
            'messages': [],
            'fields': {
                'name1': {
                    'success': True,
                    'messages': [],
                    'values': [{
                        'value': 'one',
                        'success': True,
                        'messages': [],
                    }]
                },
                'name2': {
                    'success': True,
                    'messages': [],
                    'values': []
                },
            },
        }


class ExampleFormValidator(FormValidator):

    message = 'example validator'

    def __init__(self):
        super().__init__()
        self._validate = True

    def validate(self):
        return self._validate



from pytest import fixture
from mock import create_autospec

from formskit.translation import Translation, Translable
from formskit.tests.base import FormskitTestCase


class TranslationTests(FormskitTestCase):

    def test_casting(self):
        class InheritedTranslation(Translation):
            pass

        message = Translation()
        message.init('text', 'one', two='two')

        casted = InheritedTranslation(message)

        assert casted.text == 'text'
        assert casted.args == ('one',)
        assert casted.kwargs == {'two': 'two'}

    def test_normaln_translating(self):
        message = Translation()
        message.init('text {0} {two}', 'one', two='two')

        assert message() == 'text one two'

    def test_inherited_translation(self):
        class InheritedTranslation(Translation):

            def translate(self):
                return self.text + 'something'

        message = Translation()
        message.init('text {0} {two}', 'one', two='two')

        casted = InheritedTranslation(message)
        assert casted() == 'text one twosomething'


class TestTranslable(object):

    @fixture
    def translable(self, message):
        obj = Translable()
        obj.messages = [message]
        return obj

    @fixture
    def message(self):
        return create_autospec(Translation)

    def test_reset(self, translable):
        """
        .reset should make empty list
        """
        translable.reset()

        assert translable.messages == []

    def test_get_error_messages(self, translable, message):
        """
        .get_error_messages should return compiled messages.
        """
        assert translable.get_error_messages() == [message.return_value]



from json import dumps
from base64 import urlsafe_b64encode

from .converters import FakeConvert
from .translation import Translable


class Field(Translable):

    def __init__(self,
                 name,
                 validators=None,
                 label=None,
                 ignore=False,
                 convert=None):
        self.name = name
        self.label = label
        self.ignore = ignore
        self.form = None
        self._init_validators(validators)
        self._init_convert(convert)
        self.values = []
        self.reset(True)
        self._avalible_values = None

    def set_avalible_values(self, avalible_values):
        self._avalible_values = avalible_values

    @property
    def avalible_values(self):
        if self._avalible_values is None:
            return []
        return (
            self._avalible_values
            if hasattr(self._avalible_values, '__contains__')
            else self._avalible_values()
        )

    def _init_validators(self, validators=None):
        self.validators = []
        validators = validators or []
        for validator in validators:
            validator.init_field(self)
            self.validators.append(validator)

    def init_form(self, form):
        """
        Set form parent.
        """
        self.form = form

    def _init_convert(self, convert):
        if convert is None:
            convert = FakeConvert()
        self.convert = convert
        self.convert._set_field(convert)

    def reset(self, force=False):
        """
        Reset field values.

        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        if self._can_this_be_edited(force):
            super().reset()
            self.values = []
            self.error = False

    def validate(self):
        """
        Validate field.

        :return: Is validation successed?
        """
        for validator in self.validators:
            validator.make_field()

        if self.error:
            return False

        for validator in self.validators:
            for value in self.values:
                validator.make_value(value)
        return not self.error

    def set_error(self, text):
        """
        Sets error for field.

        :param text: text of error
        """
        self.error = True
        message = self._get_message_object()
        message.init(text, field=self)
        self.messages.append(message)

    def get_value(self, index=0, default=NotImplemented):
        """
        Gets value from field.

        :param index: index of value
        :param default: what will be return if value is not found. If not set,
            then raise IndexError
        """
        try:
            field_value = self.values[index]
        except IndexError:
            if default is NotImplemented:
                raise
            else:
                return default
        return self.convert(field_value.value)

    def get_value_errors(self, index=0, default=NotImplemented):
        """
        Gets errors from value.

        :param index: index of value
        :param default: what will be return if value is not found. If not set,
            then raise IndexError
        """
        try:
            field_value = self.values[index]
        except IndexError:
            if default is NotImplemented:
                raise
            else:
                return default
        return field_value.get_error_messages()

    def get_values(self):
        """
        Get list of values from field.
        """
        return [
            self.convert(field_value.value)
            for field_value in self.values
        ]

    def _can_this_be_edited(self, force):
        return force is True or self.ignore is False

    def set_values(self, values, force=False):
        """
        Sets values for field. This method will remove old ones.

        :param values: list of values to set
        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        if not self._can_this_be_edited(force):
            return
        self.values = []
        for value in values:
            self.values.append(
                FieldValue(
                    self,
                    value
                )
            )

    def set_value(self, value, index=0, force=False):
        """
        Sets value at index or at the end of list of values.

        :param index: index of value
        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        if not self._can_this_be_edited(force):
            return
        try:
            field_value = self.values[index]
            field_value.value = self.convert.back(value)
        except IndexError:
            self.values.append(
                FieldValue(
                    self,
                    self.convert.back(value)
                ))

    def get_name(self):
        """
        Get name of field.
        """
        return self.name

    def _get_message_object(self, *args, **kwargs):
        return self.form._get_message_object(*args, **kwargs)


class TreeField(Field):

    def get_name(self):
        """
        Get name of field.
        """
        data = {
            'name': self.name,
            'parents': self.form._get_parents(),
        }
        json = dumps(data)
        return urlsafe_b64encode(json.encode())


class FieldValue(Translable):

    def __init__(self, field, value):
        self.field = field
        self.value = value
        self.reset()

    def set_error(self, text):
        """
        Set error for field value.

        :param text: text of error
        """
        self.error = True
        self.field.error = True
        message = self.field._get_message_object()
        message.init(text, field=self.field, value=self)
        self.messages.append(message)

    def reset(self):
        super().reset()
        self.error = False



from base64 import urlsafe_b64decode
from copy import deepcopy
from json import loads
import binascii

from .field import TreeField
from .form import Form, WrongValueName


class TreeForm(Form):

    def _init_before_create(self):
        super()._init_before_create()
        self.parent = None
        self.childs = {}

    def add_field(self, *args, **kwargs):
        field = TreeField(*args, **kwargs)
        self.add_field_object(field)
        return field

    def reset(self):
        super().reset()
        for name in self.childs:
            self.childs[name] = {0: self.childs[name][0]}
            self.childs[name][0].reset()

    def _get_parents(self):
        if self.parent is None:
            return [self._get_form_info()]
        else:
            parents = self.parent._get_parents()
            parents.append(self._get_form_info())
            return parents

    def _get_form_info(self):
        return {
            'name': self.get_name(),
            'index': self.index}

    def add_sub_form(self, form):
        form._set_parent(self, 0)
        self.childs[form.get_name()] = {0: form}

    def _set_parent(self, parent, index):
        self.parent = parent
        self.index = index

    def get_or_create_sub_form(self, name, index):
        try:
            return self.get_sub_form(name, index)
        except KeyError:
            form = self._clone_sub_form(name)
            self.childs[name][index] = form
            form.index = index
            return form

    def get_sub_form(self, name, index):
        return self.childs[name][index]

    def _clone_sub_form(self, name):
        form = self.childs[name][0]
        form = deepcopy(form)
        form.reset()
        return form

    def _validate_sub_forms(self):
        success = True
        for sub_forms in self.childs.values():
            for sub_form in sub_forms.values():
                success &= sub_form._validate()
        return success

    def _parse_sub_form(self, name, data):
        for index, values in enumerate(data):
            sub_form = self.get_or_create_sub_form(name, index)
            sub_form.parse_dict(values)

    def _validate(self):
        # Why this method was implemented in this way?
        # Goal was to run validation on fields and if it succeeded, then form
        # can run form validators.
        # But sub_forms should always run validation.
        super()._validate()
        self.success &= self._validate_sub_forms()
        return self.success

    def _get_field(self, name):
        data = self._decode_name(name)
        name = data['name']

        form = self._get_sub_form(data['parents'][1:])
        return form.fields[name]

    def get_data_dict(self, minified=False):
        tree = super().get_data_dict(minified=minified)
        for name, sub_forms in self.childs.items():
            tree[name] = {}
            for index, sub_form in sub_forms.items():
                tree[name][index] = sub_form.get_data_dict(minified)
        return tree

    def _decode_name(self, name):
        try:
            json = urlsafe_b64decode(name).decode('utf8')
            return loads(json)
        except (binascii.Error, UnicodeDecodeError):
            raise WrongValueName(name)

    def _get_sub_form(self, parents):
        if parents == []:
            return self
        form_name = parents[0]['name']
        index = parents[0]['index']
        sub_form = self.get_or_create_sub_form(form_name, index)
        return sub_form._get_sub_form(parents[1:])

    def get_report(self):
        reports = super().get_report()
        reports['childs'] = {}
        for name, forms in self.childs.items():
            reports['childs'][name] = {}
            for index, form in forms.items():
                reports['childs'][name][index] = form.get_report()
        return reports



from .field import Field
from .formvalidators import FormValidationError
from .translation import Translation, Translable


class Form(Translable):

    form_name_value = 'form_name'
    translation_class = Translation

    def get_name(self):
        """Gets name of this form."""
        return self.__class__.__name__

    def __init__(self):
        self._init_before_create()
        self.create_form()
        self.reset()

    def _init_before_create(self):
        self.fields = {}
        self.form_validators = []
        self.raw_data = None
        self.index = None

    def add_field_object(self, field):
        """
        Add field to form.

        :param field: field object
        """
        self.fields[field.name] = field
        field.init_form(self)

    def add_field(self, *args, **kwargs):
        """
        Create and add field to form.
        Params are the same as in ``Field`` init.
        """
        field = Field(*args, **kwargs)
        self.add_field_object(field)
        return field

    def validate(self, raw_data):
        """
        Gets raw data from dict and validate this object.

        :returns:
            * None: if form is not submited
            * True: if form is submited, and validation is success
            * Fail: if form is submited, and validation is fails
        """
        if self._is_form_submitted(raw_data):
            self._parse_raw_data(raw_data)
            if self._validate():
                self.on_success()
                return True
            else:
                self.on_fail()
                return False
        else:
            return None

    def _is_form_submitted(self, raw_data):
        return raw_data.get(self.form_name_value, None) == [self.get_name(), ]

    def _parse_raw_data(self, raw_data):
        for name, values in raw_data.items():
            if name == self.form_name_value:
                continue
            field = self._get_field(name)
            field.set_values(values)

    def _get_field(self, name):
        try:
            return self.fields[name]
        except KeyError:
            raise WrongValueName(name)

    def reset(self):
        """Reset the form and clear all it's fields."""
        super().reset()
        self.success = None
        for field in self.fields.values():
            field.reset()

    def _validate(self):
        # Why this method was implemented in this way?
        # Goal was to run validation on fields and if it succeeded, then form
        # can run form validators.
        # But sub_forms should always run validation.
        self.success = True
        self.success &= (
            self._validate_fields()
            and self._validate_form_validators()
        )
        return self.success

    def _validate_fields(self):
        success = True
        for field in self.fields.values():
            success &= field.validate()
        return success

    def _validate_form_validators(self):
        for validator in self.form_validators:
            try:
                validator()
            except FormValidationError as er:
                message = self._get_message_object()
                message.init(er.message, form=self)
                self.messages.append(message)
                return False
        return True

    def add_form_validator(self, validator):
        """
        Adds form validator to form.

        :param validator: Object of FormValidator class.
        """
        validator.set_form(self)
        self.form_validators.append(validator)

    def get_values(self, name):
        """
        Get all values of field.

        :param name: name of field.
        :rtype: FieldValue
        """
        return self.fields[name].get_values()

    def get_value(self, name, index=0, default=NotImplemented):
        """
        Get field value at index.

        :param name: name of field
        :param index: index of value in field (default: 0)
        :param default: what to return if value not found (default: raise
            error)
        """
        return self.fields[name].get_value(index, default)

    def set_values(self, name, values, force=False):
        """
        Set values for field.

        :param name: name of field
        :param values: list of values
        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        self.fields[name].set_values(values, force=force)

    def set_value(self, name, value, index=0, force=False):
        """set_value(name, value[, index, force])
        Set value at index.

        :param name: name of field
        :param value: value
        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        self.fields[name].set_value(value, index, force=force)

    def get_data_dict(self, minified=False):
        """get_data_dict([minified])
        Get all values from all fields.

        :param minified: if True: all list with length of 1 will be converted
            to direct value. all empty list will be omited
        :rtype: dict
        """
        tree = {}
        for name, field in self.fields.items():
            tree[name] = field.get_values()
            if minified and len(tree[name]) == 1:
                tree[name] = tree[name][0]
            elif minified and len(tree[name]) == 0:
                del tree[name]
        return tree

    def parse_dict(self, data, force=False):
        """parse_dict(data[, force])
        Parse data from dict. Keys is the field name, values can be in form of
        list or without it.

        :param data: dict of data to parse
        :param force: if set to False and field.ignore is set to True, nothing
            will happend
        """
        for name, values in data.items():
            if name in self.fields:
                field = self.fields[name]
                if hasattr(values, '__iter__') and type(values) is not str:
                    field.set_values(values, force=force)
                else:
                    field.set_value(values, force=force)
            else:
                self._parse_sub_form(name, values)

    def get_report(self):
        """Get report from all fields."""
        def convert(messages):
            data = []
            for message in messages:
                data.append(message())
            return data

        report = {
            'success': self.success,
            'messages': convert(self.messages),
            'fields': {},
        }
        for name, field in self.fields.items():
            values = []
            for value in field.values:
                values.append({
                    'value': value.value,
                    'success': not value.error,
                    'messages': convert(value.messages),
                })
            report['fields'][name] = {
                'success': not field.error,
                'messages': convert(field.messages),
                'values': values,
            }
        return report

    def _get_message_object(self, *args, **kwargs):
        return self.translation_class(*args, **kwargs)

    def create_form(self):
        """
        This metod will be called to create form.
        It should be reimplemented.
        """
        pass

    def on_success(self):
        """
        This metod will be called when validation will success.
        It should be reimplemented.
        """
        pass

    def on_fail(self):
        """
        This metod will be called when validation will fail.
        It should be reimplemented.
        """
        pass


class WrongValueName(Exception):

    def __init__(self, name):
        self.name = name
